import os
import mock
import requests
import yaml
import builtins
import obs.libs.gmt as gmt
from obs.libs import config
from obs.libs import auth as auth_lib


def fake_policies_file():
    return "gmt_policy.yaml"


def fake_config():
    pass


def test_policies_file(monkeypatch):
    monkeypatch.setattr(config, "load_config_file", fake_config)
    monkeypatch.setattr(os.environ, "get", lambda name: fake_policies_file())
    assert gmt.policies_file() == "gmt_policy.yaml"


def test_exists(monkeypatch):
    monkeypatch.setattr(gmt, "policies_file", fake_policies_file)
    monkeypatch.setattr(os.path, "isfile", lambda file: True)
    assert gmt.is_policy_exists() is True


def fake_get_policies():
    return {
        "WJV-1": {"id": "1c32d6320", "description": "", "scheme": {"DC1": "3"}},
        "Dual-DC": {
            "id": "d3031d77b",
            "description": "2 Replication in Midplaza, 1 in Technovillage",
            "scheme": {"DC2": "2", "DC1": "2"},
        },
    }


def fake_notset():
    return "notset"


def fake_open(file):
    pass


def test_policies(monkeypatch):
    monkeypatch.setattr(gmt, "policies_file", fake_policies_file)
    monkeypatch.setattr(gmt, "is_policy_exists", lambda: True)
    monkeypatch.setattr(builtins, "open", fake_open)
    monkeypatch.setattr(yaml, "safe_load", lambda file: fake_get_policies())

    assert gmt.get_policies() == (
        {
            "WJV-1": {"id": "1c32d6320", "description": "", "scheme": {"DC1": "3"}},
            "Dual-DC": {
                "id": "d3031d77b",
                "description": "2 Replication in Midplaza, 1 in Technovillage",
                "scheme": {"DC2": "2", "DC1": "2"},
            },
        }
    )


def test_notset(monkeypatch):
    monkeypatch.setattr(gmt, "policies_file", fake_notset)
    assert gmt.get_policies() == "notset"


def fake_request(bucket_name, auth):
    response = mock.Mock()
    response.headers.get.return_value = "dd7e84cfe467c0fc11b5b075ac9acd73"
    return response


def fake_auth():
    return ("<requests_aws4auth>", "new.net")


def test_policy_id(monkeypatch):
    monkeypatch.setattr(requests, "get", fake_request)
    monkeypatch.setattr(auth_lib, "get_endpoint", lambda url, bucket_name: None)
    assert "dd7e84cfe467c0fc11b5b075ac9acd73" == gmt.policy_id(
        "awesome-bucket", fake_auth()
    )


def test_policy_description(monkeypatch):
    monkeypatch.setattr(gmt, "get_policies", fake_get_policies)
    assert (
        gmt.policy_description("d3031d77b")
        == f"2 Replication in Midplaza, 1 in Technovillage"
    )


def test_description_notset(monkeypatch):
    monkeypatch.setattr(gmt, "get_policies", fake_notset)
    assert gmt.policy_description("d3031d77b") is None


def test_no_description(monkeypatch):
    monkeypatch.setattr(gmt, "get_policies", fake_get_policies)
    assert "No description" == gmt.policy_description("1c32d6320")



import os
import dotenv
import errno
import click
import io
import sys
import pathlib


class Config:
    """Accommodate config file creation by setting and
    getting it's class variables."""

    user_access_key = ""
    user_secret_key = ""
    user_url = "nos.wjv-1.neo.id"
    user_gmt_policy = "notset"
    admin_url = ""
    admin_port = ""
    admin_username = ""
    admin_password = ""
    use_https = "true"
    use_neo = "false"

    def dump_config(self, options, cfg):
        cfg_file = config_file()
        config = ""
        for option in options:
            value = getattr(cfg, option[0])

            option = f"OBS_{option[0].upper()}"
            config += f"{option}={value}\n"
        try:
            create_config_dir()
            with io.open(cfg_file, "w") as fp:
                fp.write(config)
                click.secho(f"\nConfiguration saved to {cfg_file}", fg="green")
        except IOError as e:
            click.secho(
                f"\nWriting config file failed: {cfg_file}: {e.strerror}",
                fg="yellow",
                bold=True,
                err=True,
            )
            sys.exit()


def create_config_dir():
    home = os.path.expanduser("~")
    config_dir = os.path.join(home, ".config", "neo-obs")
    pathlib.Path(config_dir).mkdir(parents=True, exist_ok=True)


def config_file():
    home = os.path.expanduser("~")
    cfg_file = os.path.join(home, ".config", "neo-obs", "obs.env")
    return cfg_file


def is_config_exists():
    cfg_file = config_file()
    return os.path.isfile(cfg_file)


def load_config_file():
    cfg_file = config_file()
    # load_dotenv didn't have it's own exception
    if is_config_exists():
        dotenv.load_dotenv(cfg_file, override=True)
    else:
        raise FileNotFoundError(errno.ENOENT, os.strerror(errno.ENOENT), cfg_file)



import re

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

with open("obs/__init__.py", "r", encoding="utf8") as f:
    version = re.search(r'__version__ = "(.*?)"', f.read()).group(1)

with open("README.rst", "rb") as f:
    readme = f.read().decode("utf-8")

with open("requirements.txt", "rb") as f:
    requirements = f.read().decode("utf-8")

setup(
    name="neo-obs",
    version=version,
    description="A OBS command line tools",
    long_description=readme,
    long_description_content_type="text/x-rst",
    url="https://github.com/BiznetGIO/neo-obs",
    author="BiznetGio",
    author_email="support@biznetgio.com",
    license="MIT license",
    classifiers=[
        "Intended Audience :: Developers",
        "Topic :: Utilities",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.7",
    ],
    keywords="cli",
    include_package_data=True,
    packages=["obs"],
    install_requires=requirements,
    entry_points={"console_scripts": ["obs=obs.cli.main:cli"]},
)



import click
import sys
from functools import partial

from obs.libs import auth
from obs.cli.admin import user as user_cli
from obs.cli.admin import qos as qos_cli
from obs.cli.admin import credential as cred_cli
from obs.cli.admin import usage as usage_cli
from obs.libs import config

click.option = partial(click.option, show_default=True)


def get_admin_client():
    try:
        config.load_config_file()
        admin_client = auth.admin_client()
        return admin_client
    except Exception as exc:
        click.secho(str(exc), fg="yellow", bold=True, err=True)
        msg = (
            f"Configuration file not available.\n"
            f"Consider running 'obs --configure' to create one"
        )
        click.secho(msg, fg="yellow", bold=True, err=True)
        sys.exit(1)


@click.group()
def admin():
    """administrate object storage."""


@admin.group()
def user():
    """administrate user."""


@user.command("ls")
@click.option("--group-id", "group_id", type=str, help="Group ID")
@click.option("--user-type", "user_type", default="all", help="User Type")
@click.option("--user-status", "user_status", default="active", help="User Status")
@click.option("--limit", "limit", default="", help="Maximum data length")
def list(group_id, user_type, user_status, limit):
    """List users."""
    client = get_admin_client()
    user_cli.list_user(
        client=client,
        group_id=group_id,
        user_type=user_type,
        user_status=user_status,
        limit=limit,
    )


@user.command("info")
@click.option("--user-id", "user_id", type=str, help="User ID")
@click.option("--group-id", "group_id", type=str, help="Group ID")
def info(user_id, group_id):
    """Get user info."""
    client = get_admin_client()
    user_cli.info(client=client, user_id=user_id, group_id=group_id)


@user.command("create")
def create():
    """Create user."""
    client = get_admin_client()
    user_cli.create(client)


@user.command("suspend")
@click.option("--user-id", "user_id", type=str, help="User ID")
@click.option("--group-id", "group_id", type=str, help="Group ID")
@click.option(
    "--suspend/--unsuspend",
    "suspend",
    default=True,
    help="suspend or unsuspend spesific user",
)
def suspend(user_id, group_id, suspend):
    """suspend user"""
    client = get_admin_client()
    status = True if suspend else False
    user_cli.suspend(client=client, user_id=user_id, group_id=group_id, status=status)


@user.command("rm")
@click.option("--user-id", "user_id", type=str, help="User ID")
@click.option("--group-id", "group_id", type=str, help="Group ID")
def rm(user_id, group_id):
    """Remove user"""
    client = get_admin_client()
    user_cli.remove(client=client, user_id=user_id, group_id=group_id)


@admin.group()
def qos():
    """administrate QoS."""


@qos.command("info")
@click.option("--user-id", "user_id", type=str, help="User ID")
@click.option("--group-id", "group_id", type=str, help="Group ID")
def info(user_id, group_id):
    """Get QoS info of specified user"""
    client = get_admin_client()
    qos_cli.info(client, user_id=user_id, group_id=group_id)


@qos.command("set")
@click.option("--user-id", "user_id", type=str, help="User ID")
@click.option("--group-id", "group_id", type=str, help="Group ID")
@click.option("--limit", "limit", type=int, help="Storage limit")
def set(user_id, group_id, limit):
    """Set QoS of specified user"""
    client = get_admin_client()
    qos_cli.set(client, user_id=user_id, group_id=group_id, limit=limit)


@qos.command("rm")
@click.option("--user-id", "user_id", type=str, help="User ID")
@click.option("--group-id", "group_id", type=str, help="Group ID")
def rm(user_id, group_id):
    """Remove QoS of specified user"""
    client = get_admin_client()
    qos_cli.rm(client, user_id=user_id, group_id=group_id)


@admin.group()
def cred():
    """administrate user credentials."""


@cred.command("ls")
@click.option("--user-id", "user_id", type=str, help="User ID")
@click.option("--group-id", "group_id", type=str, help="Group ID")
def ls(user_id, group_id):
    """Show user's credentials."""
    client = get_admin_client()
    cred_cli.list(client, user_id, group_id)


@cred.command("status")
@click.option("--access-key", "access_key", type=str, help="User Access Key")
@click.option("--status", "status", default=True, help="Set Status")
def status(access_key, status):
    """Set user's credentials status."""
    client = get_admin_client()
    cred_cli.status(client, access_key, status)


@cred.command("rm")
@click.option("--access-key", "access_key", type=str, help="User Access Key")
def rm(access_key):
    """Remove user's credentials."""
    client = get_admin_client()
    cred_cli.rm(client, access_key)


@cred.command("create")
@click.option("--user-id", "user_id", type=str, help="User ID")
@click.option("--group-id", "group_id", type=str, help="Group ID")
def create(user_id, group_id):
    """Create user's credentials."""
    client = get_admin_client()
    cred_cli.create(client, user_id, group_id)


@admin.group()
def usage():
    """Administrate usage."""


@usage.command("du")
@click.option("--user-id", "user_id", type=str, help="User ID")
@click.option("--group-id", "group_id", type=str, help="Group ID")
def du(user_id, group_id):
    """Show user's usage."""
    client = get_admin_client()
    usage_cli.du(client, user_id, group_id)



import click

from obs.libs import credential as cred_lib
from obs.libs import utils


def list(client, user_id, group_id):
    try:
        credentials = cred_lib.list(client, user_id, group_id)
        utils.check(credentials)
        for credential in credentials:
            human_date = utils.human_date(credential["createDate"])
            result = (
                f"Access Key: {credential['accessKey']}\n"
                f"Secret Key: {credential['secretKey']}\n"
                f"Created: {human_date}\n"
                f"Active: {credential['active']}"
            )
            click.secho(result)
    except Exception as exc:
        click.secho(
            f"Credentials listing failed. \n{exc}", fg="yellow", bold=True, err=True
        )


def status(client, access_key, status):
    try:
        status = cred_lib.status(client, access_key, status)
        utils.check(status)
        click.secho(f"Credentials status changed", fg="green")
    except Exception as exc:
        click.secho(
            f"Credentials fetching failed. \n{exc}", fg="yellow", bold=True, err=True
        )


def rm(client, access_key):
    try:
        credentials = cred_lib.rm(client, access_key)
        utils.check(credentials)
        click.secho(f"Credentials removed", fg="green")
    except Exception as exc:
        click.secho(
            f"Credential removal failed. \n{exc}", fg="yellow", bold=True, err=True
        )


def create(client, user_id, group_id):
    try:
        credentials = cred_lib.create(client, user_id, group_id)
        utils.check(credentials)
        click.secho(f"Credentials created", fg="green")
    except Exception as exc:
        click.secho(
            f"Credential creation failed. \n{exc}", fg="yellow", bold=True, err=True
        )



import mock
import boto3
from obs.libs import auth
from obs.libs import config


def fake_config():
    pass


def fake_session(**kwargs):
    session = mock.Mock()
    session.resource.return_value = "s3_resource"
    return session


def test_resource(monkeypatch):
    monkeypatch.setattr(config, "load_config_file", fake_config)
    monkeypatch.setattr(boto3, "Session", fake_session)
    monkeypatch.setattr(auth, "get_endpoint", lambda url: None)
    assert auth.resource() == "s3_resource"
